@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import division
+import math
 import imposm.geom
 
 ANY = '__any__'
@@ -26,6 +28,7 @@ __all__ = [
     'ZOrder',
     'PointTable',
     'String',
+    'LocalizedName',
     'LineStringTable',
     'Direction',
     'OneOfInt',
@@ -34,8 +37,46 @@ __all__ = [
     'Bool',
     'GeneralizedTable',
     'UnionView',
+    'set_default_name_field',
 ]
 
+default_name_field = None
+
+def set_default_name_type(type, column_name='name'):
+    """
+    Set new default type for 'name' field.
+    
+    ::
+        
+        set_default_name_type(LocalizedName(['name:en', 'int_name', 'name']))
+    
+    """
+    global default_name_field
+    default_name_field = column_name, type
+
+# changed by imposm.app if the projection is epsg:4326
+import_srs_is_geographic = False
+
+def meter_to_mapunit(meter):
+    """
+    Convert ``meter`` into the mapunit of the import.
+    Only supports EPSG:4326 (degrees) at the moment, all other
+    SRS will use meter as mapunit.
+    """
+    if import_srs_is_geographic:
+        deg_to_meter = (40000 * 1000) / 360
+        return meter / deg_to_meter
+    return meter
+
+def sqr_meter_to_mapunit(sqr_meter):
+    """
+    Convert ``sqr_meter`` into the mapunit of the import.
+    Only supports EPSG:4326 (degrees) at the moment, all other
+    SRS will use meter as mapunit.
+    """
+    if import_srs_is_geographic:
+        return meter_to_mapunit(math.sqrt(sqr_meter))**2
+    return sqr_meter
 
 class Mapping(object):
     table = None
@@ -44,21 +85,23 @@ class Mapping(object):
     classname = None
     _insert_stmt = None
     
-    filter_out_names = set((
-        'fixme', 'fix me', 'fix-me!',
-        'kein name', 'kein',
-        'unbenannt', 'unbekannt',
-        'noch unbekannt', 'noch ohne namen',
-        'noname', 'unnamed', 'namenlos', 'no_name', 'no name',
-    ))
-    
     def __init__(self, name, mapping, fields=None, field_filter=None):
         self.name = name
         self.mapping = mapping
-        if fields:
-            self.fields = fields
+        self.fields = fields or tuple(self.fields)
+        self._add_name_field()
         if field_filter:
             self.field_filter = field_filter
+
+    def _add_name_field(self):
+        """
+        Add name field to default if not set.
+        """
+        if not any(1 for name, _type in self.fields if name == 'name'):
+            if default_name_field:
+                self.fields = (default_name_field,) + self.fields
+            else:
+                self.fields = (('name', Name()),) + self.fields
         
     @property
     def insert_stmt(self):
@@ -66,12 +109,18 @@ class Mapping(object):
             self._insert_stmt = self.table('osm_' + self.name, self).insert_stmt
         return self._insert_stmt
     
-    def name_filter(self, osm_elem):
-        if osm_elem.name and osm_elem.name.lower() in self.filter_out_names:
-            osm_elem.name = ''
-    
     def extra_field_names(self):
-        return [n for n,_ in self.fields] + [n for n,_ in self.field_filter]
+        extra_field_names = []
+        for field_name, field_filter in self.field_filter:
+            extra_field_names.append(field_name)
+        
+        for field_name, field in self.fields:
+            field_names = field.extra_fields()
+            if field_names is not None:
+                extra_field_names.extend(field_names)
+            else:
+                extra_field_names.append(field_name)
+        return extra_field_names
     
     def build_geom(self, osm_elem):
         try:
@@ -134,7 +183,6 @@ class TagMapper(object):
         return self._mapping_for_tags(self.polygon_mappings, tags)
 
     def _tag_filter(self, filter_tags):
-        filter_tags['name'] = set(['__any__'])
         def filter(tags):
             for k in tags.keys():
                 if k not in filter_tags:
@@ -275,6 +323,16 @@ class DropElem(Exception):
 
 
 class FieldType(object):
+    def extra_fields(self):
+        """
+        List with names of fields (keys) that should be processed
+        during read-phase.
+        
+        Return ``None`` to use the field name from the mapping.
+        Return ``[]`` if no extra fields (keys) are required.
+        """
+        return None
+    
     def value(self, val, osm_elem):
         return val
 
@@ -286,6 +344,60 @@ class String(FieldType):
     """
     column_type = "VARCHAR(255)"
 
+class Name(String):
+    """
+    Field for name values.
+    
+    Filters out common FixMe values.
+    
+    :PostgreSQL datatype: VARCHAR(255)
+
+    .. versionadded:: 2.3.0
+    """
+    
+    filter_out_names = set((
+        'fixme', 'fix me', 'fix-me!',
+        '0', 'none', 'n/a', 's/n',
+        'kein name', 'kein',
+        'unbenannt', 'unbekannt',
+        'noch unbekannt', 'noch ohne namen',
+        'noname', 'unnamed', 'namenlos', 'no_name', 'no name',
+        'editme', '_edit_me_',
+    ))
+
+    def value(self, val, osm_elem):
+        if val and val.lower() in self.filter_out_names:
+            osm_elem.name = ''
+            val = ''
+        return val
+
+class LocalizedName(Name):
+    """
+    Field for localized name values.
+    Checks different name keys and uses the first key with
+    a valid value.
+    
+    :param coalesce: list of name keys to check
+    :PostgreSQL datatype: VARCHAR(255)
+    
+    .. versionadded:: 2.3.0
+    """
+    def __init__(self, coalesce=['name', 'int_name']):
+        self.coalesce_keys = coalesce
+    
+    def extra_fields(self):
+        return self.coalesce_keys
+    
+    def value(self, val, osm_elem):
+        for key in self.coalesce_keys:
+            val = osm_elem.tags.get(key)
+            if val and val.lower() not in self.filter_out_names:
+                osm_elem.name = val
+                return val
+        else:
+            osm_elem.name = ''
+            return ''
+
 class Bool(FieldType):
     """
     Field for boolean values.
@@ -334,6 +446,42 @@ class Direction(FieldType):
                 return -1
         return 0
 
+class PseudoArea(FieldType):
+    """
+    Field for the (pseudo) area of a polygon in square meters.
+    
+    The value is just an approximation since the geometries are in
+    EPSG:4326 and not in a equal-area projection. The approximation
+    is good for smaller polygons (<1%) and should be precise enough
+    to compare geometries for rendering order (larger below smaller).
+    
+    The area of the geometry is multiplied by the cosine of
+    the mid-latitude to compensate the reduced size towards
+    the poles.
+    
+    :PostgreSQL datatype: REAL
+    
+    .. versionadded:: 2.3.0
+    """
+    
+    column_type = "REAL"
+    
+    def value(self, val, osm_elem):
+        area = osm_elem.geom.area
+        if not area:
+            return None
+        
+        extent = osm_elem.geom.bounds
+        mid_lat = extent[1] + (abs(extent[3] - extent[1]) / 2)
+        sqr_deg = area * math.cos(math.radians(mid_lat))
+        
+        # convert deg^2 to m^2
+        sqr_m = (math.sqrt(sqr_deg) * (40075160 / 360))**2
+        return sqr_m
+    
+    def extra_fields(self):
+        return []
+
 class OneOfInt(FieldType):
     """
     Field type for integer values.
@@ -383,6 +531,9 @@ class ZOrder(FieldType):
         for i, t in enumerate(types[::-1]):
             self.rank[t] = i
 
+    def extra_fields(self):
+        return []
+
     def value(self, val, osm_elem):
         return self.rank.get(osm_elem.type, 0)
 
@@ -419,6 +570,9 @@ class WayZOrder(FieldType):
     }
 
     brunnel_bool = Bool()
+    
+    def extra_fields(self):
+        return []
 
     def value(self, val, osm_elem):
         tags = osm_elem.tags
